@@ -24,6 +24,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import itertools
 import os
 import pprint
+import random
 import re
 import signal
 import subprocess
@@ -42,7 +43,9 @@ except ImportError:
 
 from flent import aggregators, formatters, resultset, loggers
 from flent.metadata import record_metadata, record_postrun_metadata
-from flent.util import clean_path, format_date
+from flent.util import clean_path, format_date, token_split
+from flent.settings import parser as SETTINGS_PARSER
+from flent.settings import _LOG_DEFER
 
 # Python2/3 compatibility
 try:
@@ -164,8 +167,8 @@ class BatchRunner(object):
         return ret.replace("$$", "$")
 
     def apply_args(self, values, args={}, settings=None):
-        new = OrderedDict(args)
-        new.update(values)
+        new = OrderedDict(values)
+        new.update(args)
         for k, v in new.items():
             new[k] = self.interpolate(v, new, settings)
 
@@ -174,7 +177,7 @@ class BatchRunner(object):
     def commands_for(self, batch, settings=None):
         if 'commands' not in batch:
             return []
-        cmdnames = [i.strip() for i in batch['commands'].split(',')]
+        cmdnames = [i.strip() for i in token_split(batch['commands'])]
         commands = OrderedDict()
 
         while cmdnames:
@@ -192,7 +195,7 @@ class BatchRunner(object):
 
             # Commands can specify extra commands to run; expand those, use the
             # dictionary to prevent duplicates
-            extra = [i.strip() for i in cmd.get('extra_commands', '').split(',')
+            extra = [i.strip() for i in token_split(cmd.get('extra_commands', ''))
                      if i.strip()]
             for e in reversed(extra):
                 cmdnames.insert(0, e)
@@ -265,9 +268,14 @@ class BatchRunner(object):
         return clean_path(filename)
 
     def expand_argsets(self, batch, argsets, batch_time, batch_name,
-                       print_status=True):
+                       print_status=True, no_shuffle=False):
 
-        for argset in itertools.product(*argsets):
+        sets = itertools.product(*argsets)
+        if self.settings.BATCH_SHUFFLE and not no_shuffle:
+            sets = list(sets)
+            random.shuffle(sets)
+
+        for argset in sets:
             rep = argset[-1]
             argset = argset[:-1]
             settings = self.settings.copy()
@@ -295,7 +303,7 @@ class BatchRunner(object):
 
             settings.load_rcvalues(b.items(), override=True)
             settings.NAME = b['test_name']
-            settings.load_test(informational=settings.BATCH_DRY)
+            settings.load_test(informational=True)
             settings.DATA_FILENAME = self.gen_filename(settings, b, argset, rep)
 
             yield b, settings
@@ -306,7 +314,7 @@ class BatchRunner(object):
         for k in batch.keys():
             if k.startswith("for_"):
                 argset = []
-                for a in batch[k].split(','):
+                for a in token_split(batch[k]):
                     a = a.strip().lower()
                     matches = [arg for arg in self.args if fnmatch(arg, a)]
                     if not matches:
@@ -333,7 +341,8 @@ class BatchRunner(object):
         argsets = self.get_argsets(batch)
 
         for _, s in self.expand_argsets(batch, argsets, self.settings.TIME,
-                                        batch_name, False):
+                                        batch_name, print_status=False,
+                                        no_shuffle=True):
             total_time += s.TOTAL_LENGTH + int(batch.get('pause', 0))
             n += 1
 
@@ -422,13 +431,14 @@ class BatchRunner(object):
                 self.logfile = loggers.setup_logfile(
                     os.path.join(output_path,
                                  "%s.log" % settings.DATA_FILENAME),
-                    level=loggers.INFO)
-            if b.get('debug_log', False):
-                self.logfile_debug = loggers.setup_logfile(
-                    os.path.join(output_path,
-                                 "%s.debug.log" % settings.DATA_FILENAME),
-                    level=loggers.DEBUG,
-                    maxlevel=loggers.DEBUG)
+                    level=loggers.INFO, replay=False)
+                if b.get('debug_log', False):
+                    self.logfile_debug = loggers.setup_logfile(
+                        os.path.join(output_path,
+                                     "%s.debug.log" % settings.DATA_FILENAME),
+                        level=loggers.DEBUG,
+                        maxlevel=loggers.DEBUG,
+                        replay=False)
 
             if settings.DATA_FILENAME in filenames_seen:
                 logger.warning("Filename already seen in this run: %s",
@@ -445,12 +455,15 @@ class BatchRunner(object):
                         logger.info("  Running test '%s'.", settings.NAME)
                     logger.info("   data_filename=%s", settings.DATA_FILENAME)
                     for k in sorted(b.keys()):
-                        if k in settings.parser:
+                        if k.upper() in SETTINGS_PARSER:
                             logger.info("   %s=%s", k, b[k])
 
                 if settings.BATCH_DRY:
                     self.tests_run += 1
                 else:
+                    # Load test again with informational=False to enable host
+                    # lookups and other actions that may fail
+                    settings.load_test(informational=False)
                     self.run_test(settings, output_path)
             except KeyboardInterrupt:
                 self.run_commands(commands, 'post', essential_only=True)
@@ -484,6 +497,11 @@ class BatchRunner(object):
         settings = settings.copy()
         settings.load_test()
         res = resultset.new(settings)
+
+        if settings.LOG_FILE is _LOG_DEFER:
+            settings.LOG_FILE = res.dump_filename.replace(res.SUFFIX, ".log")
+            loggers.setup_logfile(settings.LOG_FILE)
+
         record_metadata(res, settings.EXTENDED_METADATA,
                         settings.REMOTE_METADATA)
 
@@ -560,6 +578,7 @@ class BatchRunner(object):
         elif self.settings.BATCH_NAMES:
             start_time = self.settings.TIME
             self.settings.BATCH_UUID = str(uuid.uuid4())
+            self.settings.LOG_FILE = None  # batch run will generate logs
             logger.info("Started batch run %s at %s.",
                         self.settings.BATCH_UUID,
                         format_date(start_time, fmt="%Y-%m-%d %H:%M:%S"))
