@@ -29,7 +29,8 @@ import warnings
 
 from flent import combiners
 from flent.util import classname, long_substr, format_date, diff_parts, \
-    Glob, Update, float_pair, keyval, comma_list, ArgParam, ArgParser
+    Glob, Update, float_pair, float_pair_noomit, keyval, comma_list, ArgParam, \
+    ArgParser
 from flent.build_info import VERSION
 from flent.loggers import get_logger
 
@@ -294,6 +295,13 @@ def add_plotting_args(parser):
         "points by this value. Can be specified multiple times, in which case "
         "each value corresponds to a data series.")
 
+    parser.add_argument(
+        "--data-cutoff",
+        action="store", type=float_pair_noomit, dest="DATA_CUTOFF",
+        help="Data cutoff interval. Cut off all data points outside this "
+        "interval before plotting. For aggregate plots, this will happen "
+        "*before* aggregation, so for instance mean values will be affected.")
+
     parser.add_argument(
         "--bounds-x",
         action="append", dest="BOUNDS_X", type=float_pair, default=[],
@@ -577,6 +585,7 @@ def new(settings, plotter=None, in_worker=False, **kwargs):
             data_config=settings.DATA_SETS,
             output=settings.OUTPUT,
             gui=settings.GUI,
+            absolute_time=settings.ABSOLUTE_TIME,
             description=settings.DESCRIPTION,
             in_worker=in_worker,
 
@@ -621,6 +630,7 @@ class Plotter(ArgParam):
                  figure=None,
                  results=None,
                  in_worker=False,
+                 absolute_time=False,
                  **kwargs):
         super(Plotter, self).__init__(**kwargs)
 
@@ -638,6 +648,7 @@ class Plotter(ArgParam):
         self.callbacks = []
         self.in_worker = in_worker
         self.combined = False
+        self.absolute_time = absolute_time
 
         self.gui = gui
         self.description = description
@@ -683,8 +694,8 @@ class Plotter(ArgParam):
                 # There is a bug in matplotlib prior to 1.4.2 where the renderer
                 # can't be pickled. Work around this by monkey-patching the
                 # renderer object as needed.
-                if a._cachedRenderer and not hasattr(a._cachedRenderer,
-                                                     "__getstate__"):
+                if getattr(a, "_cachedRenderer", None) \
+                   and not hasattr(a._cachedRenderer, "__getstate__"):
                     def getState():
                         return {'width': a._cachedRenderer.width,
                                 'height': a._cachedRenderer.height,
@@ -699,11 +710,8 @@ class Plotter(ArgParam):
         self.configs = [self.config]
 
     def verify(self):
-        for a in self.figure.axes:
-            if len(a.get_lines()) > 0:
-                return True
-
-        return False
+        lengths = [len(a.get_lines()) for a in self.figure.axes]
+        return any(lengths), lengths
 
     def axes_iter(self):
         return iter(reduce(lambda x, y: x + y, [i['axes'] for i in self.configs]))
@@ -743,6 +751,8 @@ class Plotter(ArgParam):
                                             set()))
                     rks = Glob.expand_list(s['raw_key'], all_rks)
                     for k, n in zip(rks, diff_parts(rks, "::")):
+                        if k in self.filter_series:
+                            continue
                         if 'label' in s:
                             nns.append(dict(s, raw_key=k,
                                             label="%s -- %s" % (s['label'], n)))
@@ -765,6 +775,7 @@ class Plotter(ArgParam):
             for l, s in zip(self.override_labels, new_series):
                 if l is not None:
                     s['label'] = l
+                    s['label_override'] = True
 
         return dict(config, series=new_series)
 
@@ -1060,14 +1071,14 @@ class Plotter(ArgParam):
            and not self.horizontal_legend \
            and not self.legend_placement \
            and self.legends:
+
+            # Make sure we have a renderer to get size from
+            if not getattr(self.figure, '_cachedRenderer', None):
+                self.figure.canvas.draw()
+
             legend_width = max(
                 [l.get_window_extent().width for l in self.legends])
 
-            if not legend_width:  # Legend width is not set before it's drawn
-                self.figure.canvas.draw()
-                legend_width = max(
-                    [l.get_window_extent().width for l in self.legends])
-
             canvas_width = self.figure.canvas.get_width_height()[0]
             for a in self.axes_iter():
                 # Save the original width of the axis (in the interval [0..1])
@@ -1112,19 +1123,21 @@ class Plotter(ArgParam):
                                     format_date(self.metadata['TIME']),
                                     self.metadata['LENGTH'],
                                     self.metadata['STEP_SIZE'])
-            self.btm_art.append(self.figure.text(0.5,
-                                                 0.04 / self.figure.get_figheight(),
-                                                 annotation_string,
-                                                 horizontalalignment='center',
-                                                 verticalalignment='bottom',
-                                                 fontsize=8))
+            self.btm_art.append(
+                self.figure.text(0.5,
+                                 0.04 / self.figure.get_figheight(),
+                                 annotation_string,
+                                 horizontalalignment='center',
+                                 verticalalignment='bottom',
+                                 fontsize=8))
         if self.fig_note:
-            self.btm_art.append(self.figure.text(0.0,
-                                                 0.04 / self.figure.get_figheight(),
-                                                 self.fig_note,
-                                                 horizontalalignment='left',
-                                                 verticalalignment='bottom',
-                                                 fontsize=8))
+            self.btm_art.append(
+                self.figure.text(0.0,
+                                 0.04 / self.figure.get_figheight(),
+                                 self.fig_note,
+                                 horizontalalignment='left',
+                                 verticalalignment='bottom',
+                                 fontsize=8))
 
         return titles
 
@@ -1240,13 +1253,13 @@ class Plotter(ArgParam):
             # of the *axis* range, not the data range (this may be a big
             # difference if the data range is small).
             top_scale = top_percentile * 1.01
-            axis.set_ylim(ymin=0, ymax=top_scale)
+            axis.set_ylim(0, top_scale)
         else:
             if self.log_base:
                 axis.set_yscale('log', basey=self.log_base)
-                axis.set_ylim(ymin=max(0, btm_scale), ymax=top_scale)
+                axis.set_ylim(max(0, btm_scale), top_scale)
             else:
-                axis.set_ylim(ymin=btm_scale, ymax=top_scale)
+                axis.set_ylim(btm_scale, top_scale)
 
         if self.invert_y and unit in self.inverted_units:
             axis.invert_yaxis()
@@ -1271,7 +1284,8 @@ class Plotter(ArgParam):
             try:
 
                 data = np.array(
-                    list(results.raw_series(series['data'], raw_key=raw_key)),
+                    list(results.raw_series(series['data'], raw_key=raw_key,
+                                            absolute=self.absolute_time)),
                     dtype=float).transpose()
 
                 if not len(data):
@@ -1288,12 +1302,16 @@ class Plotter(ArgParam):
                                  results.series(series['data'])),
                                 dtype=float)
 
-        if 'cutoff' in config and config['cutoff'] and data.any():
-            min_x = data[0].min() + config['cutoff'][0]
-            max_x = data[0].max() + config['cutoff'][1]
+        if data.any() and (self.data_cutoff or config.get('cutoff')):
+            start, end = self.data_cutoff or config['cutoff']
+            if self.absolute_time:
+                start += results.t0
 
-            min_idx = data[0].searchsorted(min_x, side='right')
-            max_idx = data[0].searchsorted(max_x, side='left')
+            if end <= 0:
+                end += results.meta("TOTAL_LENGTH")
+
+            min_idx = data[0].searchsorted(start, side='right')
+            max_idx = data[0].searchsorted(end, side='left')
 
             data = data[:, min_idx:max_idx]
 
@@ -1310,9 +1328,14 @@ class Plotter(ArgParam):
             l = series['smoothing']
             if l % 2 != 1:
                 l += 1
-            kern = np.ones(l, dtype=float)
-            kern /= l
-            data[1] = np.convolve(data[1], kern, mode=str('same'))
+
+            if l <= len(data[1]):
+                kern = np.ones(l, dtype=float)
+                kern /= l
+                data[1] = np.convolve(data[1], kern, mode=str('same'))
+            else:
+                logger.warn("Smoothing length longer than data series %s; "
+                            "not smoothing", series['data'])
 
         return data
 
@@ -1333,7 +1356,8 @@ class CombineManyPlotter(object):
         combiner = combiners.new(combine_mode, print_n=self.combine_print_n,
                                  filter_regexps=self.filter_regexp,
                                  filter_series=self.filter_series,
-                                 save_dir=self.combine_save_dir)
+                                 save_dir=self.combine_save_dir,
+                                 data_cutoff=self.data_cutoff)
         super(CombineManyPlotter, self).plot(
             combiner(results, config),
             config,
@@ -1409,7 +1433,8 @@ class TimeseriesPlotter(Plotter):
 
         stack = 'stacked' in config and config['stacked']
 
-        x_min = x_max = 0
+        x_min = 2**63
+        x_max = 0
 
         colours = cycle(self.colours)
 
@@ -1567,8 +1592,10 @@ class BoxPlotter(TimeseriesPlotter):
         # The median lines are red, so filter out red from the list of colours
         colours = list(
             islice(cycle([c for c in self.colours if c != 'r']), group_size))
-        series_labels = self._filter_labels([s.get('label', '') for s in series])
-
+        series_labels = [s.get('label', '') for s in series]
+        if not any([s.get('label_override', False) for s in series]):
+            series_labels = self._filter_labels(series_labels)
+        ticklabel_override = False
         for i, s in enumerate(series):
             if split_results:
                 results = split_results[i]
@@ -1615,6 +1642,9 @@ class BoxPlotter(TimeseriesPlotter):
                     bp['caps'][j * 2].set_label(r.label())
                 if len(results) > 1:
                     ticklabels.append(r.label())
+                    ticklabel_override = (ticklabel_override or
+                                          r.metadata.get('label_override',
+                                                         False))
                 if len(bp['fliers']) == group_size:
                     pyplot.setp([bp['fliers'][j]], markeredgecolor=colours[j])
                     keys = 'caps', 'whiskers'
@@ -1644,7 +1674,8 @@ class BoxPlotter(TimeseriesPlotter):
         for a, b in zip(config['axes'], self.bounds_y):
             a.set_ybound(b)
 
-        ticklabels = self._filter_labels(ticklabels)
+        if not ticklabel_override:
+            ticklabels = self._filter_labels(ticklabels)
         for i, l in enumerate(ticklabels):
             if len(l) > self._max_label_length:
                 ticklabels[i] = l[:self._max_label_length] + "..."
