@@ -538,6 +538,8 @@ class ProcessRunner(RunnerBase, threading.Thread):
                                  self.__class__.__name__,
                                  self.name, extra={'runner': self})
                     try:
+                        os.kill(self.pid, signal.SIGINT)
+                        time.sleep(0.5)
                         os.kill(self.pid, signal.SIGTERM)
                     except OSError:
                         pass
@@ -879,6 +881,9 @@ class NetperfDemoRunner(ProcessRunner):
             try:
                 # The THROUGHPUT key contains the mean value even for UDP_RR tests
                 self.metadata['MEAN_VALUE'] = float(data_dict['THROUGHPUT'])
+                if self.test == 'UDP_RR':
+                    self.metadata['MEAN_VALUE'] = transformers.rr_to_ms(self.metadata['MEAN_VALUE'])
+
                 self.metadata['ELAPSED_TIME'] = float(data_dict.get('ELAPSED_TIME', 0))
                 self.metadata['UPSTREAM_TOS'] = int(data_dict.get('LOCAL_SOCKET_TOS', 0),
                                                     base=0)
@@ -1066,17 +1071,19 @@ class RegexpRunner(ProcessRunner):
     # Parse is split into a stateless class method in _parse to be able to call
     # it from find_binary.
     def parse(self, output, error=""):
-        result, raw_values, metadata = self._parse(output)
+        result, raw_values, metadata = self._parse(output, error)
         self.raw_values = raw_values
         self.metadata.update(metadata)
         return result
 
     @classmethod
-    def _parse(cls, output):
+    def _parse(cls, output, error=None):
         result = []
         raw_values = []
         metadata = {}
         lines = output.split("\n")
+        if error:
+            lines.extend(error.split("\n"))
         for line in lines:
             for regexp in cls.regexes:
                 match = regexp.match(line)
@@ -1828,9 +1835,9 @@ class SsRunner(ProcessRunner):
         if val.endswith("Mbps"):
             return float(val[:-4])
         if val.endswith("Kbps"):
-            return float(val[:-4])*1000
+            return float(val[:-4])/1000
         if val.endswith("bps"):
-            return float(val[:-3])*10**6
+            return float(val[:-3])/10**6
         return float(val)
 
     def parse_part(self, part):
