@@ -38,7 +38,8 @@ except ImportError:
 from flent.build_info import VERSION
 from flent.testenv import TestEnvironment, TEST_PATH
 from flent.loggers import get_logger
-from flent.util import FuncAction, Update, keyval, keyval_int, ArgParser
+from flent.util import FuncAction, Update, keyval, keyval_int, ArgParser, \
+    token_split
 from flent.plotters import add_plotting_args
 from flent import loggers, util, resultset, runners
 
@@ -66,6 +67,10 @@ DEFAULT_SETTINGS = {
 DICT_SETTINGS = ('DATA_SETS', 'PLOTS')
 
 
+class _LOG_DEFER:
+    pass
+
+
 class Version(FuncAction):
 
     def __call__(*args):
@@ -107,8 +112,11 @@ class LogFile(argparse.Action):
         super(LogFile, self).__init__(option_strings, dest, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
-        loggers.setup_logfile(values)
-        setattr(namespace, self.dest, values)
+        if values is None:
+            setattr(namespace, self.dest, _LOG_DEFER)
+        else:
+            loggers.setup_logfile(values)
+            setattr(namespace, self.dest, values)
 
 
 class Debug(argparse.Action):
@@ -178,8 +186,8 @@ parser.add_argument(
 parser.add_argument(
     "-p", "--plot",
     action="store", type=unicode, dest="PLOT",
-    help="Select which plot to output for the given test (implies -f plot). "
-    "Use the --list-plots option to see available plots.")
+    help="Select which plot to output for the given test (implies -f plot if no"
+    "formatter is selected). Use the --list-plots option to see available plots.")
 
 parser.add_argument(
     "-t", "--title-extra",
@@ -268,6 +276,11 @@ parser.add_argument(
     action="store_true", dest="BATCH_VERBOSE",
     help="Be verbose during batch run: Print all commands executed.")
 
+parser.add_argument(
+    "--batch-no-shuffle",
+    action="store_false", dest="BATCH_SHUFFLE",
+    help="Do not randomise the order of test runs within each batch.")
+
 parser.add_argument(
     "--batch-repetitions", action="store", type=int, dest="BATCH_REPS",
     metavar="REPETITIONS",
@@ -367,6 +380,15 @@ test_group.add_argument(
     "netperf TCP_MAERTS and TCP_STREAM parameters, so only works for tests "
     "that employ these as their data transfer, and only for the TCP streams.")
 
+test_group.add_argument(
+    "--socket-stats",
+    action="store_true", dest="SOCKET_STATS",
+    help="Parse socket stats during test. This will capture and parse socket "
+    "statistics for all TCP upload flows during a test, adding TCP cwnd and RTT "
+    "values to the test data. Requires the 'ss' utility to be present on the "
+    "system, and can fail if there are too many simultaneous upload flows; which "
+    "is why this option is not enabled by default.")
+
 plot_group = parser.add_argument_group(
     "Plot configuration",
     "These options are used to configure the appearance of "
@@ -381,7 +403,8 @@ tool_group.add_argument(
     action="store", type=unicode, dest="CONTROL_HOST", metavar="HOST",
     help="Hostname for control connection for test tools that support it "
     "(netperf and D_ITG). If not supplied, this will be the same as the test "
-    "target host.")
+    "target host. The per-flow test parameter setting takes precedence of this "
+    "for multi-target tests.")
 
 tool_group.add_argument(
     "--control-local-bind",
@@ -436,8 +459,10 @@ misc_group = parser.add_argument_group("Misc and debugging options")
 
 misc_group.add_argument(
     "-L", "--log-file",
-    action=LogFile, type=unicode, dest="LOG_FILE",
-    help="Write debug log (test program output) to log file.")
+    action=LogFile, type=unicode, dest="LOG_FILE", nargs='?',
+    help="Write debug log (test program output) to log file. If the option is "
+    "enabled but no file name is given, the log file name is derived from the "
+    "test data filename.")
 
 misc_group.add_argument(
     '--list-tests',
@@ -545,10 +570,15 @@ class Settings(argparse.Namespace):
             elif t:
 
                 val = t(v)
+                c = parser.get_choices(k)
+                if c and val not in c:
+                    logger.warning("Invalid RC value '%s' for key %s. Ignoring",
+                                   val, k)
+                    continue
                 if isinstance(val, dict) and k in vals:
                     vals[k].update(val)
                 elif parser.is_list(k):
-                    vals[k] = [t(i.strip()) for i in v.split(",")]
+                    vals[k] = [t(i.strip()) for i in token_split(v)]
                 else:
                     vals[k] = val
 
@@ -561,7 +591,7 @@ class Settings(argparse.Namespace):
         except (ValueError, argparse.ArgumentTypeError) as e:
             raise RuntimeError("Unable to parse RC values: %s" % e)
 
-        for k, v in vals:
+        for k, v in vals.items():
             if override or getattr(self, k) == parser.get_default(k):
                 setattr(self, k, v)
 
@@ -585,6 +615,7 @@ class Settings(argparse.Namespace):
             raise RuntimeError("Missing test name.")
         test_env = TestEnvironment(self.__dict__, informational)
         filename = os.path.join(TEST_PATH, self.NAME + ".conf")
+        logger.debug("Executing test environment file %s", filename)
         s = test_env.execute(filename)
 
         for k, v in list(s.items()):
@@ -606,10 +637,11 @@ class Settings(argparse.Namespace):
             if dname not in results:
                 runner = runners.get(dvals['runner'])
                 if issubclass(runner, runners.ComputingRunner):
+                    logger.debug("Computing missing data series %s", dname)
                     try:
                         runner = runner(name=dname, settings=self,
                                         post=True, **dvals)
-                        runner.result(results)
+                        runner.compute_result(results)
                     except Exception as e:
                         logger.exception("Unable to compute missing data "
                                          "series '%s': '%s'.", dname, e)
@@ -637,8 +669,12 @@ class Settings(argparse.Namespace):
         object.__setattr__(self, k, v)
 
     def update(self, values):
+        updated = False
         for k, v in list(values.items()):
-            setattr(self, k, v)
+            if not hasattr(self, k) or getattr(self, k) != v:
+                updated = True
+                setattr(self, k, v)
+        return updated
 
     def items(self):
         return self.__dict__.items()
@@ -674,7 +710,7 @@ class Settings(argparse.Namespace):
         if self.REMOTE_METADATA:
             self.EXTENDED_METADATA = True
 
-        if self.PLOT is not None:
+        if self.PLOT is not None and self.FORMAT == 'default':
             self.FORMAT = 'plot'
 
         if self.BATCH_REPS is not None:
