@@ -52,6 +52,11 @@ except ImportError:
 SPECIAL_PARAM_NAMES = ['upload_streams', 'download_streams']
 SPECIAL_PARAM_MAP = {'num_cpus': CPU_COUNT}
 
+# Test parameters that will be parsed and from test parameters and passed to the
+# add_stream callback. In addition to these, 'stream_delay' is handled specially
+# in the code below
+STREAM_CONFIG_PARAM_NAMES = ['label', 'ping_label', 'marking',
+                             'control_host', 'local_bind', 'cc_algo']
 
 class _no_default():
     pass
@@ -83,6 +88,7 @@ class TestEnvironment(object):
             'try_test_parameters': self.try_test_parameters,
             'parse_int': self.parse_int,
             'zip_longest': zip_longest,
+            'for_stream_config': self.for_stream_config,
         })
 
         self.informational = informational
@@ -137,11 +143,15 @@ class TestEnvironment(object):
     def set_test_parameter(self, name, value):
         self.env['TEST_PARAMETERS'][name] = value
 
-    def get_test_parameter(self, name, default=_no_default, split=False):
+    def get_test_parameter(self, name, default=_no_default, split=False, cast=None):
         try:
             ret = self.env['TEST_PARAMETERS'][name]
             if split:
                 ret = token_split(ret)
+                if cast:
+                    ret = list(map(cast, ret))
+            elif cast:
+                ret = cast(ret)
             return ret
         except KeyError:
             if default is not _no_default:
@@ -186,3 +196,41 @@ class TestEnvironment(object):
             else:
                 raise RuntimeError("Need %d hosts, only %d specified" %
                                    (count, len(self.env['HOSTS'])))
+
+    def for_stream_config(self, func, n=None):
+        if n is None:
+            n = len(self.env['HOSTS'])
+
+        config_params = {}
+        for k in STREAM_CONFIG_PARAM_NAMES:
+            config_params[k] = self.get_test_parameter(k+"s",
+                                                       default=[],
+                                                       split=True)
+
+        stream_delays = self.get_test_parameter("stream_delays",
+                                                default=[],
+                                                split=True)
+        global_delay = self.env['DELAY']
+        total_length = self.env['TOTAL_LENGTH']
+        stream_length = total_length-2*global_delay
+
+        for i in range(n):
+            kwargs = {}
+            try:
+                kwargs['host'] = self.env['HOSTS'][i]
+            except IndexError:
+                pass
+
+            for k, v in config_params.items():
+                try:
+                    kwargs[k] = v[i]
+                except IndexError:
+                    pass
+
+            try:
+                kwargs['length'] = stream_length - int(stream_delays[i])
+                kwargs['delay'] = global_delay + int(stream_delays[i])
+            except IndexError:
+                pass
+
+            func(i, **kwargs)
