@@ -23,19 +23,21 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 import argparse
 import bz2
+import fnmatch
 import gzip
 import io
 import os
 import re
+import shlex
 import socket
 import time
 
-from bisect import bisect_left
 from copy import copy
 from calendar import timegm
-from datetime import datetime
-from fnmatch import fnmatch
-from math import ceil, log10, exp, sqrt
+from datetime import datetime, timedelta
+from math import log10, exp, sqrt
+
+from flent.loggers import get_logger
 
 ENCODING = "UTF-8"
 try:
@@ -49,6 +51,8 @@ try:
 except ImportError:
     import ConfigParser as configparser
 
+logger = get_logger(__name__)
+
 
 def uscore_to_camel(s):
     """Turn a underscore style string (org_table) into a CamelCase style string
@@ -81,21 +85,49 @@ def format_bytes(nbytes):
         return (nbytes, 'bytes')
 
 
-def parse_date(timestring):
+def parse_date(timestring, min_t=None, offset=None):
     try:
         # Try to parse the straight UTC time string (has a Z at the end)
-        return datetime.strptime(timestring, "%Y-%m-%dT%H:%M:%S.%fZ")
+        return datetime.strptime(timestring, "%Y-%m-%dT%H:%M:%S.%fZ"), None
     except ValueError:
         try:
             dt = datetime.strptime(timestring, "%Y-%m-%dT%H:%M:%S.%f")
         except ValueError:
             dt = datetime.strptime(timestring, "%Y-%m-%dT%H:%M:%S")
-        # The timestamp is in local time, so get the (UTC) timestamp and
-        # subtract the time zone offset at that time to get the UTC datetime
-        # object.
-        ts = time.mktime(dt.timetuple())
-        offset = datetime.fromtimestamp(ts) - datetime.utcfromtimestamp(ts)
-        return dt - offset
+
+        if offset is None:
+            # The timestamp is in local time, so we need to find the timezone
+            # offset. Guess this by computing the nearest whole-hour offset for
+            # from the minimum timestamp in the data series (passed as min_t)
+            #
+            # If not min_t is available, compute the offset from the system time
+            # zone.
+            #
+            # Return the offset so it can be re-used the next time this function
+            # is called.
+
+            logger.debug("Non-UTC timestamp %s, computing timezone offset",
+                         timestring)
+
+            if min_t is not None:
+                ts = orig_ts = timegm(dt.timetuple())
+                hours = 0
+                mult = 1 if ts < min_t else -1
+                while abs(ts - min_t) > 1800:
+                    hours -= 1 * mult
+                    ts += 3600 * mult
+                offset = timedelta(hours=hours)
+                logger.debug("Guessed timezone offset of %d hours for "
+                             "ts %f min_t %f diff %f",
+                             hours, orig_ts, min_t, orig_ts-min_t)
+            else:
+                ts = time.mktime(dt.timetuple())
+                offset = (datetime.fromtimestamp(ts) -
+                          datetime.utcfromtimestamp(ts))
+                logger.debug("Computed offset of %s from system timezone",
+                             offset)
+
+        return dt - offset, offset
 
 
 def clean_path(path, allow_dirs=False):
@@ -124,51 +156,38 @@ def long_substr(data, prefix_only=False):
     return substr
 
 
-# Calculate discrete cdf function using bisect_left.
-def cum_prob(data, val, size):
-    return bisect_left(data, val) / size
+def diff_parts(strings, sep):
+    """Return the unique parts of a set of strings by splitting on
+    a separator and pruning parts that are identical for all strings"""
 
-# from http://code.activestate.com/recipes/66472/
+    parts = [s.split(sep) for s in strings]
+    np = [p for p in zip(*parts) if len(set(p)) >= 1]
 
-
-def frange(limit1, limit2=None, increment=1.):
-    """
-    Range function that accepts floats (and integers).
-
-    Usage:
-    frange(-2, 2, 0.1)
-    frange(10)
-    frange(10, increment = 0.5)
-
-    The returned value is an iterator.  Use list(frange) for a list.
-    """
-
-    if limit2 is None:
-        limit2, limit1 = limit1, 0.
-    else:
-        limit1 = float(limit1)
-
-    count = int(ceil((limit2 - limit1) / increment))
-    return (limit1 + n * increment for n in range(count))
+    return [sep.join(p) for p in zip(*np)]
 
 
 def is_executable(filename):
     return os.path.isfile(filename) and os.access(filename, os.X_OK)
 
 
-def which(executable, fail=False):
+def which(executable, fail=None):
     pathname, filename = os.path.split(executable)
     if pathname:
         if is_executable(executable):
+            logger.debug("which: %s is a full path and executable", executable)
             return executable
     else:
         for path in [i.strip('""') for i in os.environ["PATH"].split(os.pathsep)]:
             filename = os.path.join(path, executable)
             if is_executable(filename):
+                logger.debug("which: Found %s executable at %s",
+                             executable, filename)
                 return filename
+            else:
+                logger.debug("which: %s is not an executable file", filename)
 
     if fail:
-        raise RuntimeError("No %s binary found in PATH." % executable)
+        raise fail("No %s binary found in PATH." % executable)
     return None
 
 
@@ -185,14 +204,19 @@ def path_components(path):
 
 
 def lookup_host(hostname, version=None):
+    logger.debug("Looking up hostname '%s'.", hostname)
     if version == 4:
         version = socket.AF_INET
     elif version == 6:
         version = socket.AF_INET6
     else:
         version = socket.AF_UNSPEC
-    hostnames = socket.getaddrinfo(hostname, None, version,
-                                   socket.SOCK_STREAM)
+    try:
+        hostnames = socket.getaddrinfo(hostname, None, version,
+                                       socket.SOCK_STREAM)
+    except Exception:
+        hostnames = None
+
     if not hostnames:
         raise RuntimeError("Found no hostnames on lookup of %s" % hostname)
 
@@ -327,13 +351,25 @@ class Glob(object):
             self.exclude = exclude
         self.pattern = pattern
 
+    def __repr__(self):
+        return "<Glob: %s (excl: %s)>" % (self.pattern, ",".join(self.exclude))
+
     def filter(self, values, exclude, args=None):
         if args is not None:
             pattern = self.pattern.format(**args)
         else:
             pattern = self.pattern
+
+        # Exclude * from matching :, make ** match everything
+        re_pat = fnmatch.translate(pattern)
+        re_pat = re_pat.replace(".*.*", ":::PLACEHOLDER:::")
+        re_pat = re_pat.replace(".*", "[^:]*")
+        re_pat = re_pat.replace(":::PLACEHOLDER:::", ".*")
+        regex = re.compile(re_pat)
+
         exclude += self.exclude
-        return [x for x in values if fnmatch(x, pattern) and x not in exclude]
+        return (x for x in values if regex.match(x) and
+                not any((fnmatch.fnmatch(x, e) for e in exclude)))
 
     def __iter__(self):
         return iter((self,))  # allow list(g) to return [g]
@@ -414,7 +450,7 @@ def mos_score(T, loss):
         Idd = 0
     else:
         Idd = 25 * ((1 + X**6)**(1 / 6) - 3 *
-                    (1 + (X / 3)**6)**1 / 6 + 2)  # (7-27)
+                    (1 + (X / 3)**6)**(1 / 6) + 2)  # (7-27)
 
     Idle = (Ro - Rle) / 2 + sqrt((Ro - Rle)**2 / 4 + 169)  # (7-25)
 
@@ -481,13 +517,21 @@ def float_pair(value):
         raise argparse.ArgumentTypeError("Invalid pair value: %s" % value)
 
 
+def float_pair_noomit(value):
+    val = float_pair(value)
+    if None in val:
+        raise argparse.ArgumentTypeError("Invalid pair value: %s" % value)
+    return val
+
+
 def keyval(value):
     ret = {}
-    for p in value.split(";"):
-        if not p.strip():
+    for p in token_split(value, ";"):
+        p = p.strip()
+        if not p:
             continue
         try:
-            k, v = p.split('=', 1)
+            k, v = token_split(p, '=')
             ret.update({k.strip(): v.strip()})
         except ValueError:
             raise argparse.ArgumentTypeError(
@@ -504,11 +548,21 @@ def keyval_int(value):
 
 def comma_list(value):
     try:
-        return [v.strip() for v in value.split(",")]
+        return [v.strip() for v in token_split(value)]
     except ValueError:
         raise argparse.ArgumentTypeError("Unable to split into list.")
 
 
+def token_split(value, split_tokens=',;'):
+    """Split VALUE on the tokens given in SPLIT_TOKENS, while
+    avoiding splitting of quoted strings"""
+
+    lex = shlex.shlex(value, posix=True)
+    lex.whitespace_split = True
+    lex.whitespace = split_tokens
+    return list(lex)
+
+
 class ArgParam(object):
     """A class that takes an argparser and sets object properties from
     the argparser-defined parameters."""
@@ -539,6 +593,13 @@ class ArgParser(argparse.ArgumentParser):
                 return action.type
         return None
 
+    def get_choices(self, dest):
+        for action in self._actions:
+            if action.dest == dest and action.choices:
+                return action.choices
+
+        return None
+
     def is_list(self, dest):
         for action in self._actions:
             if action.dest == dest:
